/**
 * userSettings.js
 * Copyright (C) 2007-2011 Tommi Rautava
 * 
 * This file is part of Popomungo.
 *
 * Popomungo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Popomungo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

org.mozdev.popomungo.UserSettings = {
	
	characterIdUrlRegExp: /[&?]CharacterID=([0-9]+)/i,
	
	companyIdUrlRegExp: /[&?]LocaleID=([0-9]+)/i,
	
	locationUrlRegExp: /\/Locale\.asp/i,
	
	charId: 0,	
	langId: 0,	
	companyId: 0,	
	dateFormatTypeId: 0,
	timeFormatTypeId: 0,
	
	CHARACTER_ID_XPATH: "//tr[@class='MainMenu']/td[1]/a[2]",

	COMPANY_ID_XPATH: "//tr[@class='MainMenu']/td[1]/a[5]",

	//CITY_NAME_ON_CITY_PAGE_XPATH: '/html/body/table[3]/tbody/tr/td[1]/div[3]/b',
	
	CITY_NAME_ON_CHARACTER_PAGE_XPATH: 
		"/html/body/table[3]/tbody/tr/td[1]/table[1]/tbody/tr/td[2]/table/tbody/tr[2]/td/a[contains(@href, 'CityID')]/b",

	CHARACTER_NAME_ON_CHARACTER_PAGE_XPATH:
		'/html/body/table[3]/tbody/tr/td[1]/table[1]/tbody/tr/td[2]/table/tbody/tr[1]/td[1]/b',
		
	CHARACTER_PORTRAIT_ON_CHARACTER_PAGE_XPATH:
		"//div[@id='pic']//img",
		

	_readCharacterId: function _readCharacterId(doc) 
	{
		try {
			var link1 = org.mozdev.popomungo.commonFunctions.XPathFirstNode(this.CHARACTER_ID_XPATH, doc);

			if (link1) {
				var matchArray = link1.search.match(this.characterIdUrlRegExp);
					
				if (matchArray) {
					this.charId = parseInt(matchArray[1]);
				} else {
					org.mozdev.popomungo.logger.debug("No character ID:\npage="+ doc.location +"\nlink="+ link1.href);
				}
			} else {
				org.mozdev.popomungo.logger.debug("No character ID:\npage="+ doc.location);
			}
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}

		org.mozdev.popomungo.logger.debug('charId='+ this.charId);

		return this.charId;
	},
	
	
	_readCEOCompanyId: function _readCEOCompanyId(doc) 
	{
		try {
			var link1 = org.mozdev.popomungo.commonFunctions.XPathFirstNode(this.COMPANY_ID_XPATH, doc);

			if (link1) {
				var matchArray = link1.search.match(this.companyIdUrlRegExp);
					
				if (matchArray) {
					this.companyId = parseInt(matchArray[1]);
					org.mozdev.popomungo.logger.debug('companyId='+ this.companyId);
				}
				else if (link1.search.match(this.locationUrlRegExp)) {
					this.companyId = 0;
					org.mozdev.popomungo.logger.debug('Not CEO');
				}
				else {
					org.mozdev.popomungo.logger.debug("No company ID:\npage="+ doc.location +"\nlink="+ link1.href);
				}
			} else {
				org.mozdev.popomungo.logger.debug("No company ID:\npage="+ doc.location);
			}
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}

		return this.companyId;
	},
	
	
	getCharacterId: function getCharacterId(aDocument) {
		return this._readCharacterId(aDocument);
	},
	
	
	getCEOCompanyId: function (aDocument) {
		return this._readCEOCompanyId(aDocument);
	},
	
	
	getLanguageId: function getLanguageId() 
	{
		try {
			if (this.langId) {
				return this.langId;
			}
			
			this.langId = parseInt(org.mozdev.popomungo.prefs.getCharacterPref(
				this.charId, 
				"extensions.popomungo.characters.<n>.languageId"));
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}
		
		return this.langId;
	},
	

	getDateFormatTypeId: function getDateFormatTypeId() 
	{
		try {
			if (this.dateFormatTypeId) {
				return this.dateFormatTypeId;
			}

			this.dateFormatTypeId = parseInt(org.mozdev.popomungo.prefs.getCharacterPref(
				this.charId, 
				"extensions.popomungo.characters.<n>.dateFormatTypeId"));
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}
		
		return this.dateFormatTypeId;
	},
	

	updateSettings: function updateSettings(doc, formElements) 
	{
		try {
			if (org.mozdev.popomungo.prefs.isEnabled("extensions.popomungo.options.autoDetectLanguage.enabled")) {
				this.langId = parseInt(formElements.namedItem("LanguageID").value);
				
				org.mozdev.popomungo.prefs.setCharacterPref(
					this.charId, 
					"extensions.popomungo.characters.<n>.languageId", 
					this.langId);
			}
			
			if (org.mozdev.popomungo.prefs.isEnabled("extensions.popomungo.options.autoDetectDateAndTimeFormat.enabled")) {
				this.dateFormatTypeId = parseInt(formElements.namedItem("DateFormatTypeID").value);
			
				org.mozdev.popomungo.prefs.setCharacterPref(
					this.charId, 
					"extensions.popomungo.characters.<n>.dateFormatTypeId", 
					this.dateFormatTypeId);

				this.timeFormatTypeId = parseInt(formElements.namedItem("TimeFormatTypeID").value);
			
				org.mozdev.popomungo.prefs.setCharacterPref(
					this.charId, 
					"extensions.popomungo.characters.<n>.timeFormatTypeId", 
					this.timeFormatTypeId);					
			}
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}
	},
	

	updateSettingsOnLoad: function updateSettingsOnLoad(doc) 
	{
		try {
			org.mozdev.popomungo.logger.debug("Updating configuration data from the user settings");
		
			// capture the onsubmit event on the form
			doc.forms[0].addEventListener('submit', org.mozdev.popomungo.UserSettings, true);
		
			this.updateSettings(doc, doc.forms[0].elements);
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}
	},
	
	
	handleEvent: function handleEvent(event) {
		try {
			org.mozdev.popomungo.logger.debug("Updating configuration data from the user settings");
			var doc = event.originalTarget.ownerDocument;
			org.mozdev.popomungo.logger.debug('doc='+ doc);
			org.mozdev.popomungo.UserSettings.updateSettings(doc, doc.forms[0].elements);
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}
	},
	

	autoDetectLanguageFromPage:
	function autoDetectLanguageFromPage(doc) {
		try {
			this.getLanguageId();
			
			if (org.mozdev.popomungo.prefs.isEnabled("extensions.popomungo.options.autoDetectLanguage.enabled")) {
				var detectedLangId = org.mozdev.popomungo.popoLang.detectLanguage(doc, this.langId);
				
				if (detectedLangId && 
					(this.langId != detectedLangId)) 
				{
					this.langId = detectedLangId;
					
					org.mozdev.popomungo.prefs.setCharacterPref(
						this.getCharacterId(doc), 
						"extensions.popomungo.characters.<n>.languageId",
						detectedLangId);
				}
			}
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}
		
		return this.langId;
	},
	

	updateSettingsFromCharacterPage:
	function updateSettingsFromCharacterPage(doc) {
		try {
			var charIdMatch = doc.location.search.match(this.characterIdUrlRegExp);				
			
			if (charIdMatch) {
				var pageCharId = parseInt(charIdMatch[1]);
				var charId = this.getCharacterId(doc); 
				
				if (charId != pageCharId) {
					return;
				}
			}
			else {
				// OK, assume that the current page is the
				// character details page of the own character.
				org.mozdev.popomungo.logger.debug("No character ID on the URL");
			}

			// Parse city name.
			if (org.mozdev.popomungo.prefs.isEnabled("extensions.popomungo.options.autoDetectCityName.enabled")) {
				var cityNameNode = org.mozdev.popomungo.commonFunctions.XPathFirstNode(
					this.CITY_NAME_ON_CHARACTER_PAGE_XPATH, doc);
	
				if (cityNameNode) {
					org.mozdev.popomungo.prefs.setCharacterPref(
						this.charId, 
						"extensions.popomungo.characters.<n>.cityName", 
						cityNameNode.textContent);
				}
			}
			
			// Parse character name.
			var characterNameNode = org.mozdev.popomungo.commonFunctions.XPathFirstNode(
				this.CHARACTER_NAME_ON_CHARACTER_PAGE_XPATH, doc);
			
			if (characterNameNode) {
				org.mozdev.popomungo.prefs.setCharacterPref(
					this.charId, 
					"extensions.popomungo.characters.<n>.name", 
					characterNameNode.textContent);
			}
			
			// Parse character portrait.
			var characterPortraitNode = org.mozdev.popomungo.commonFunctions.XPathFirstNode(
				this.CHARACTER_PORTRAIT_ON_CHARACTER_PAGE_XPATH, doc);
				
			if (characterPortraitNode) {
				org.mozdev.popomungo.prefs.setCharacterPref(
					this.charId, 
					"extensions.popomungo.characters.<n>.portraitUrl", 
					characterPortraitNode.src);
			}
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}
	},
	
	
	updateSettingsFromCityPage: 
	function updateSettingsFromCityPage(doc) 
	{
		try {
			/*
			var cityNameNode = org.mozdev.popomungo.commonFunctions.XPathFirstNode(this.CITY_NAME_ON_CITY_PAGE_XPATH, doc);
				
			org.mozdev.popomungo.prefs.setCharacterPref(
				this.charId, 
				"extensions.popomungo.characters.<n>.cityName", 
				cityNameNode.textContent);
			*/
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}
	},
	
	
	LoggedOut:
	function LoggedOut(doc) {
		this.charId = 0;
		this.langId = 0;
		this.companyId = 0;
		this.dateFormatTypeId = 0;
		this.timeFormatTypeId = 0;
	},
	
	
	IsOnline:
	function IsOnline() {
		return (this.charId != 0);
	},
	
	
	GetCharacterIdList: function GetCharacterIdList() {
		var res = new Array();
		
		try {
			var children = org.mozdev.popomungo.prefs.getChildList("extensions.popomungo.characters.");
			var idList = {};

			var len = 0;
			var tmp = {};
			
			// Gather unique values.
			for (var i = children.length - 1; i > -1 ; i--) {
				var id = children[i].split(".")[0];
				
				tmp[id] = true;
			}

			var j = 0;
			
			// Collect the results.
			for (var t in tmp) {
				res[j++] = t;
			}
			
			res.sort(function (a, b) { return (parseInt(a) - parseInt(b)); });
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}
		
		return res;
	},
	
	
	observe: function observe(aSubject, aTopic, aData)
	{
		org.mozdev.popomungo.logger.debug('topic='+ aTopic +', data='+ aData);
		
		if (aTopic == 'nsPref:changed') {
			var charId = org.mozdev.popomungo.UserSettings.charId;
			
			if (!charId) {
				org.mozdev.popomungo.logger.debug("Character ID not defined");
				return;	
			}
			
			org.mozdev.popomungo.logger.debug("Character ID="+ charId);
			
			if (aData == "extensions.popomungo.characters.<n>.cityName".replace("<n>", charId)) {
				org.mozdev.popomungo.logger.debug("City name updated");
			}
			else if (aData == "extensions.popomungo.characters.<n>.portraitUrl".replace("<n>", charId)) {
				org.mozdev.popomungo.logger.debug("Character portrait URL updated");
			}
			else if (aData == "extensions.popomungo.characters.<n>.languageId".replace("<n>", charId)) {
				org.mozdev.popomungo.logger.debug("Language ID reset");
				org.mozdev.popomungo.UserSettings.langId = 0;
			}
			else if (aData == "extensions.popomungo.characters.<n>.name".replace("<n>", charId)) {
				org.mozdev.popomungo.logger.debug("Character name updated");
			}
			else if (aData == "extensions.popomungo.characters.<n>.dateFormatTypeId".replace("<n>", charId)) {
				org.mozdev.popomungo.logger.debug("Date format type ID reset");
				org.mozdev.popomungo.UserSettings.dateFormatTypeId = 0;
			}
			else if (aData == "extensions.popomungo.characters.<n>.timeFormatTypeId".replace("<n>", charId)) {
				org.mozdev.popomungo.logger.debug("Time format type ID reset");
				org.mozdev.popomungo.UserSettings.timeFormatTypeId = 0;
			}
			else {
				org.mozdev.popomungo.logger.debug("No match found: "+ aData);
			}
		}
	}
};
